"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var path_1 = __importDefault(require("path"));
var FAKE_MODULE_NAME = 'webpack-inject-plugin.module.js';
var FAKE_LOADER_NAME = 'webpack-inject-plugin.loader.js';
var ID = 1;
exports.registry = {};
function getUniqueID() {
    return "webpack-inject-module-" + ++ID;
}
function injectToArray(originalEntry, newEntry, entryOrder) {
    if (entryOrder === void 0) { entryOrder = 2 /* NotLast */; }
    if (entryOrder === 0 /* First */) {
        return [newEntry].concat(originalEntry);
    }
    if (entryOrder === 1 /* Last */) {
        return originalEntry.concat([newEntry]);
    }
    return originalEntry.splice(0, originalEntry.length - 1).concat([
        newEntry
    ], originalEntry.splice(originalEntry.length - 1));
}
function injectEntry(originalEntry, newEntry, options) {
    if (originalEntry === undefined) {
        return newEntry;
    }
    // Last module in an array gets exported, so the injected one must not be
    // last. https://webpack.github.io/docs/configuration.html#entry
    if (typeof originalEntry === 'string') {
        return injectToArray([originalEntry], newEntry, options.entryOrder);
    }
    if (Array.isArray(originalEntry)) {
        return injectToArray(originalEntry, newEntry, options.entryOrder);
    }
    if (originalEntry === Object(originalEntry)) {
        return Object.entries(originalEntry).reduce(function (a, _a) {
            var key = _a[0], entry = _a[1];
            if (!options.entryName ||
                (options.entryName && options.entryName === key)) {
                a[key] = injectEntry(entry, newEntry, options);
            }
            else {
                a[key] = entry;
            }
            return a;
        }, {});
    }
    return originalEntry;
}
exports.injectEntry = injectEntry;
var WebpackInjectPlugin = /** @class */ (function () {
    function WebpackInjectPlugin(loader, options) {
        this.loader = loader;
        this.options = {
            entryName: (options && options.entryName) || undefined,
            entryOrder: (options && options.entryOrder) || 2 /* NotLast */
        };
    }
    WebpackInjectPlugin.prototype.apply = function (compiler) {
        var moduleLocation = path_1.default.join(__dirname, FAKE_MODULE_NAME);
        var id = getUniqueID();
        exports.registry[id] = this.loader;
        compiler.options.entry = injectEntry(compiler.options.entry, moduleLocation, this.options);
        if (compiler.options.module === undefined) {
            compiler.options.module = {
                rules: []
            };
        }
        if (compiler.options.module.rules === undefined) {
            compiler.options.module.rules = [];
        }
        compiler.options.module.rules.push({
            test: /webpack-inject-plugin\.module\.js/,
            use: [
                {
                    loader: path_1.default.join(__dirname, FAKE_LOADER_NAME),
                    query: { id: id }
                }
            ]
        });
    };
    return WebpackInjectPlugin;
}());
exports.default = WebpackInjectPlugin;
//# sourceMappingURL=main.js.map